@@ -17,17 +17,18 @@
 #include <ctype.h>
 #include <netinet/in.h>
 #include <curses.h>
-#include <dirent.h>
-#include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/fsuid.h>
-#include <urcu.h>
 #include <libgen.h>
 #include <inttypes.h>
 #include <poll.h>
 #include <fcntl.h>
 #include <arpa/inet.h>
 
+#include <urcu.h>
+#include <urcu/list.h>
+#include <urcu/rculist.h>
+
 #include "ui.h"
 #include "die.h"
 #include "xmalloc.h"
@@ -38,7 +39,6 @@
 #include "lookup.h"
 #include "geoip.h"
 #include "built_in.h"
-#include "locking.h"
 #include "pkt_buff.h"
 #include "screen.h"
 #include "proc.h"
@@ -52,7 +52,28 @@
 #define USEC_PER_SEC 1000000L
 #endif
 
+struct proc_entry {
+	struct cds_list_head entry;
+	struct cds_list_head flows;
+	struct rcu_head rcu;
+
+	struct timeval last_update;
+	unsigned int pid;
+	char name[256];
+	uint64_t pkts_src, bytes_src;
+	uint64_t pkts_dst, bytes_dst;
+	double rate_bytes_src;
+	double rate_bytes_dst;
+	double rate_pkts_src;
+	double rate_pkts_dst;
+	int flows_count;
+};
+
 struct flow_entry {
+	struct cds_list_head proc_head;
+	struct cds_list_head entry;
+	struct rcu_head rcu;
+
 	uint32_t flow_id, use, status;
 	uint8_t  l3_proto, l4_proto;
 	uint32_t ip4_src_addr, ip4_dst_addr;
@@ -66,10 +87,8 @@ struct flow_entry {
 	char country_code_src[4], country_code_dst[4];
 	char city_src[128], city_dst[128];
 	char rev_dns_src[256], rev_dns_dst[256];
-	char procname[256];
-	struct flow_entry *next;
+	struct proc_entry *proc;
 	int inode;
-	unsigned int procnum;
 	bool is_visible;
 	struct nf_conntrack *ct;
 	struct timeval last_update;
@@ -80,8 +99,11 @@ struct flow_entry {
 };
 
 struct flow_list {
-	struct flow_entry *head;
-	struct spinlock lock;
+	struct cds_list_head head;
+};
+
+struct proc_list {
+	struct cds_list_head head;
 };
 
 enum flow_direction {
@@ -128,10 +150,13 @@ static volatile bool do_reload_flows;
 static volatile bool is_flow_collecting;
 static volatile sig_atomic_t sigint = 0;
 static int what = INCLUDE_IPV4 | INCLUDE_IPV6 | INCLUDE_TCP;
+static struct proc_list proc_list;
 static struct flow_list flow_list;
 static struct sysctl_params_ctx sysctl = { -1, -1 };
 
 static unsigned int cols, rows;
+static WINDOW *screen;
+static int skip_lines;
 
 static unsigned int interval = 1;
 static bool show_src = false;
@@ -153,7 +178,23 @@ enum tbl_flow_col {
 	TBL_FLOW_RATE,
 };
 
+enum tbl_proc_col {
+	TBL_PROC_NAME,
+	TBL_PROC_PID,
+	TBL_PROC_FLOWS,
+	TBL_PROC_BYTES_SRC,
+	TBL_PROC_RATE_SRC,
+	TBL_PROC_BYTES_DST,
+	TBL_PROC_RATE_DST,
+};
+
 static struct ui_table flows_tbl;
+static struct ui_table procs_tbl;
+
+enum tab_entry {
+	TAB_FLOWS,
+	TAB_PROCS,
+};
 
 static const char *short_options = "vhTUsDIS46ut:nGb";
 static const struct option long_options[] = {
@@ -362,10 +403,16 @@ static inline void flow_entry_xfree(struct flow_entry *n)
 	xfree(n);
 }
 
+static void flow_entry_xfree_rcu(struct rcu_head *head)
+{
+	struct flow_entry *n = container_of(head, struct flow_entry, rcu);
+
+	flow_entry_xfree(n);
+}
+
 static inline void flow_list_init(struct flow_list *fl)
 {
-	fl->head = NULL;
-	spinlock_init(&fl->lock);
+	CDS_INIT_LIST_HEAD(&fl->head);
 }
 
 static inline bool nfct_is_dns(const struct nf_conntrack *ct)
@@ -376,7 +423,7 @@ static inline bool nfct_is_dns(const struct nf_conntrack *ct)
 	return ntohs(port_src) == 53 || ntohs(port_dst) == 53;
 }
 
-static void flow_list_new_entry(struct flow_list *fl, const struct nf_conntrack *ct)
+static int flow_list_new_entry(struct flow_list *fl, struct nf_conntrack *ct)
 {
 	struct flow_entry *n;
 
@@ -384,170 +431,129 @@ static void flow_list_new_entry(struct flow_list *fl, const struct nf_conntrack
 	 * use it to resolve reverse dns.
 	 */
 	if (nfct_is_dns(ct))
-		return;
+		return NFCT_CB_CONTINUE;
 
 	n = flow_entry_xalloc();
 
-	n->ct = nfct_clone(ct);
+	n->ct = ct;
 
 	flow_entry_update_time(n);
 	flow_entry_from_ct(n, ct);
 	flow_entry_get_extended(n);
 
-	rcu_assign_pointer(n->next, fl->head);
-	rcu_assign_pointer(fl->head, n);
+	cds_list_add_rcu(&n->entry, &fl->head);
 
 	n->is_visible = true;
+
+	return NFCT_CB_STOLEN;
 }
 
-static struct flow_entry *flow_list_find_id(struct flow_list *fl,
-					    uint32_t id)
+static struct flow_entry *flow_list_find_id(struct flow_list *fl, uint32_t id)
 {
-	struct flow_entry *n = rcu_dereference(fl->head);
+	struct flow_entry *n;
 
-	while (n != NULL) {
+	cds_list_for_each_entry_rcu(n, &fl->head, entry) {
 		if (n->flow_id == id)
 			return n;
-
-		n = rcu_dereference(n->next);
 	}
 
 	return NULL;
 }
 
-static struct flow_entry *flow_list_find_prev_id(const struct flow_list *fl,
-						 uint32_t id)
+static int flow_list_del_entry(struct flow_list *fl, const struct nf_conntrack *ct)
 {
-	struct flow_entry *prev = rcu_dereference(fl->head), *next;
+	struct flow_entry *n;
 
-	if (prev->flow_id == id)
-		return NULL;
-
-	while ((next = rcu_dereference(prev->next)) != NULL) {
-		if (next->flow_id == id)
-			return prev;
-
-		prev = next;
-	}
-
-	return NULL;
-}
-
-static void flow_list_destroy_entry(struct flow_list *fl,
-				    const struct nf_conntrack *ct)
-{
-	struct flow_entry *n1, *n2;
-	uint32_t id = nfct_get_attr_u32(ct, ATTR_ID);
-
-	n1 = flow_list_find_id(fl, id);
-	if (n1) {
-		n2 = flow_list_find_prev_id(fl, id);
-		if (n2) {
-			rcu_assign_pointer(n2->next, n1->next);
-			n1->next = NULL;
-
-			flow_entry_xfree(n1);
-		} else {
-			struct flow_entry *next = fl->head->next;
-
-			flow_entry_xfree(fl->head);
-			fl->head = next;
+	n = flow_list_find_id(fl, nfct_get_attr_u32(ct, ATTR_ID));
+	if (n) {
+		if (n->proc) {
+			cds_list_del(&n->proc_head);
+			n->proc->flows_count--;
 		}
+
+		cds_list_del_rcu(&n->entry);
+		call_rcu(&n->rcu, flow_entry_xfree_rcu);
 	}
+
+	return NFCT_CB_CONTINUE;
 }
 
 static void flow_list_destroy(struct flow_list *fl)
 {
-	struct flow_entry *n;
+	struct flow_entry *n, *tmp;
 
-	synchronize_rcu();
-	spinlock_lock(&flow_list.lock);
-
-	while (fl->head != NULL) {
-		n = rcu_dereference(fl->head->next);
-		fl->head->next = NULL;
-
-		flow_entry_xfree(fl->head);
-		rcu_assign_pointer(fl->head, n);
+	cds_list_for_each_entry_safe(n, tmp, &fl->head, entry) {
+		cds_list_del_rcu(&n->entry);
+		call_rcu(&n->rcu, flow_entry_xfree_rcu);
 	}
-
-	spinlock_unlock(&flow_list.lock);
 }
 
-static int walk_process(unsigned int pid, struct flow_entry *n)
+static void proc_list_init(struct proc_list *proc_list)
 {
-	int ret;
-	DIR *dir;
-	struct dirent *ent;
-	char path[1024];
-
-	if (snprintf(path, sizeof(path), "/proc/%u/fd", pid) == -1)
-		panic("giant process name! %u\n", pid);
-
-	dir = opendir(path);
-	if (!dir)
-        	return 0;
-
-	while ((ent = readdir(dir))) {
-		struct stat statbuf;
-
-		if (snprintf(path, sizeof(path), "/proc/%u/fd/%s",
-			     pid, ent->d_name) < 0)
-			continue;
-
-		if (stat(path, &statbuf) < 0)
-			continue;
-
-		if (S_ISSOCK(statbuf.st_mode) && (ino_t) n->inode == statbuf.st_ino) {
-			char cmdline[256];
-
-			ret = proc_get_cmdline(pid, cmdline, sizeof(cmdline));
-			if (ret < 0)
-				panic("Failed to get process cmdline: %s\n", strerror(errno));
-
-			if (snprintf(n->procname, sizeof(n->procname), "%s", basename(cmdline)) < 0)
-				n->procname[0] = '\0';
-			n->procnum = pid;
-			closedir(dir);
-			return 1;
-		}
-	}
-
-	closedir(dir);
-	return 0;
+	CDS_INIT_LIST_HEAD(&proc_list->head);
 }
 
-static void walk_processes(struct flow_entry *n)
+static struct proc_entry *proc_list_new_entry(unsigned int pid)
 {
-	int ret;
-	DIR *dir;
-	struct dirent *ent;
+	struct proc_entry *proc;
 
-	/* n->inode must be set */
-	if (n->inode <= 0) {
-		n->procname[0] = '\0';
+	cds_list_for_each_entry(proc, &proc_list.head, entry) {
+		if (proc->pid && proc->pid == pid)
+			return proc;
+	}
+
+	proc = xzmalloc(sizeof(*proc));
+
+	bug_on(gettimeofday(&proc->last_update, NULL));
+	CDS_INIT_LIST_HEAD(&proc->flows);
+	proc->pid = pid;
+
+	cds_list_add_tail(&proc->entry, &proc_list.head);
+
+	return proc;
+}
+
+static void proc_entry_xfree_rcu(struct rcu_head *head)
+{
+	struct proc_entry *p = container_of(head, struct proc_entry, rcu);
+
+	xfree(p);
+}
+
+static void proc_list_destroy(struct proc_list *pl)
+{
+	struct proc_entry *p, *tmp;
+
+	cds_list_for_each_entry_safe(p, tmp, &pl->head, entry) {
+		cds_list_del_rcu(&p->entry);
+		call_rcu(&p->rcu, proc_entry_xfree_rcu);
+	}
+}
+
+static void flow_entry_find_process(struct flow_entry *n)
+{
+	struct proc_entry *p;
+	char cmdline[512];
+	pid_t pid;
+	int ret;
+
+	ret = proc_find_by_inode(n->inode, cmdline, sizeof(cmdline), &pid);
+	if (ret <= 0)
 		return;
-	}
 
-	dir = opendir("/proc");
-	if (!dir)
-		panic("Cannot open /proc: %s\n", strerror(errno));
+	p = proc_list_new_entry(pid);
 
-	while ((ent = readdir(dir))) {
-		const char *name = ent->d_name;
-		char *end;
-		unsigned int pid = strtoul(name, &end, 10);
+	if (snprintf(p->name, sizeof(p->name), "%s", basename(cmdline)) < 0)
+		p->name[0] = '\0';
 
-		/* not a PID */
-		if (pid == 0 && end == name)
-			continue;
+	p->pkts_src += n->pkts_src;
+	p->pkts_dst += n->pkts_dst;
+	p->bytes_src += n->bytes_src;
+	p->bytes_dst += n->bytes_dst;
+	p->flows_count++;
 
-		ret = walk_process(pid, n);
-		if (ret > 0)
-			break;
-	}
-
-	closedir(dir);
+	cds_list_add(&n->proc_head, &p->flows);
+	n->proc = p;
 }
 
 static int get_port_inode(uint16_t port, int proto, bool is_ip6)
@@ -596,6 +602,19 @@ static int get_port_inode(uint16_t port, int proto, bool is_ip6)
 
 static void flow_entry_from_ct(struct flow_entry *n, const struct nf_conntrack *ct)
 {
+	uint64_t bytes_src = nfct_get_attr_u64(ct, ATTR_ORIG_COUNTER_BYTES);
+	uint64_t bytes_dst = nfct_get_attr_u64(ct, ATTR_REPL_COUNTER_BYTES);
+	uint64_t pkts_src  = nfct_get_attr_u64(ct, ATTR_ORIG_COUNTER_PACKETS);
+	uint64_t pkts_dst  = nfct_get_attr_u64(ct, ATTR_REPL_COUNTER_PACKETS);
+
+	/* Update stats diff to the related process entry */
+	if (n->proc) {
+		n->proc->pkts_src += pkts_src - n->pkts_src;
+		n->proc->pkts_dst += pkts_dst - n->pkts_dst;
+		n->proc->bytes_src += bytes_src - n->bytes_src;
+		n->proc->bytes_dst += bytes_dst - n->bytes_dst;
+	}
+
 	CP_NFCT(l3_proto, ATTR_ORIG_L3PROTO, 8);
 	CP_NFCT(l4_proto, ATTR_ORIG_L4PROTO, 8);
 
@@ -663,7 +682,7 @@ flow_entry_geo_city_lookup_generic(struct flow_entry *n,
 {
 	struct sockaddr_in sa4;
 	struct sockaddr_in6 sa6;
-	const char *city = NULL;
+	char *city = NULL;
 
 	switch (n->l3_proto) {
 	default:
@@ -687,6 +706,8 @@ flow_entry_geo_city_lookup_generic(struct flow_entry *n,
 		        sizeof(n->city_src));
 	else
 		SELFLD(dir, city_src, city_dst)[0] = '\0';
+
+	free(city);
 }
 
 static void
@@ -810,7 +831,7 @@ static void flow_entry_get_extended(struct flow_entry *n)
 	n->inode = get_port_inode(n->port_src, n->l4_proto,
 				  n->l3_proto == AF_INET6);
 	if (n->inode > 0)
-		walk_processes(n);
+		flow_entry_find_process(n);
 }
 
 static char *bandw2str(double bytes, char *buf, size_t len)
@@ -999,10 +1020,11 @@ static void draw_flow_entry(const struct flow_entry *n)
 	ui_table_row_add(&flows_tbl);
 
 	/* Application */
-	ui_table_row_col_set(&flows_tbl, TBL_FLOW_PROCESS, n->procname);
+	ui_table_row_col_set(&flows_tbl, TBL_FLOW_PROCESS,
+			      n->proc ?  n->proc->name : "");
 
 	/* PID */
-	slprintf(tmp, sizeof(tmp), "%.d", n->procnum);
+	slprintf(tmp, sizeof(tmp), "%.d", n->proc ? n->proc->pid : 0);
 	ui_table_row_col_set(&flows_tbl, TBL_FLOW_PID, tmp);
 
 	/* L4 protocol */
@@ -1093,45 +1115,10 @@ static inline bool presenter_flow_wrong_state(struct flow_entry *n)
 	return true;
 }
 
-static void draw_flows(WINDOW *screen, struct flow_list *fl,
-		       int skip_lines)
+static void draw_filter_status(char *title, int count, int skip_lines)
 {
-	int row_width = show_src ? 2 : 1;
-	unsigned int flows = 0;
-	unsigned int line = 4;
-	int skip = skip_lines;
-	struct flow_entry *n;
-
-	rcu_read_lock();
-
-	n = rcu_dereference(fl->head);
-	if (!n)
-		mvwprintw(screen, line, 2, "(No sessions! "
-			  "Is netfilter running?)");
-
-	ui_table_clear(&flows_tbl);
-	ui_table_header_print(&flows_tbl);
-
-	for (; n; n = rcu_dereference(n->next)) {
-		if (!n->is_visible)
-			continue;
-		if (presenter_flow_wrong_state(n))
-			continue;
-
-		/* count only flows which might be showed */
-		flows++;
-
-		if (line + row_width >= rows)
-			continue;
-		if (--skip >= 0)
-			continue;
-
-		draw_flow_entry(n);
-		line += row_width;
-	}
-
 	mvwprintw(screen, 1, 0, "%*s", COLS - 1, " ");
-	mvwprintw(screen, 1, 2, "Kernel netfilter flows(%u) for ", flows);
+	mvwprintw(screen, 1, 2, "%s(%u) for ", title, count);
 
 	if (what & INCLUDE_IPV4)
 		printw("IPv4,");
@@ -1157,7 +1144,116 @@ static void draw_flows(WINDOW *screen, struct flow_list *fl,
 	if (is_flow_collecting)
 		printw(" [Collecting flows ...]");
 
+}
+
+static void draw_flows(WINDOW *screen, struct flow_list *fl,
+		       int skip_lines)
+{
+	int row_width = show_src ? 2 : 1;
+	unsigned int flows = 0;
+	unsigned int line = 4;
+	int skip = skip_lines;
+	struct flow_entry *n;
+
+	rcu_read_lock();
+
+	if (cds_list_empty(&fl->head))
+		mvwprintw(screen, line, 2, "(No sessions! "
+			  "Is netfilter running?)");
+
+	ui_table_clear(&flows_tbl);
+	ui_table_header_print(&flows_tbl);
+
+	cds_list_for_each_entry_rcu(n, &fl->head, entry) {
+		if (!n->is_visible)
+			continue;
+		if (presenter_flow_wrong_state(n))
+			continue;
+
+		/* count only flows which might be showed */
+		flows++;
+
+		if (line + row_width >= rows)
+			continue;
+		if (--skip >= 0)
+			continue;
+
+		draw_flow_entry(n);
+		line += row_width;
+	}
+
 	rcu_read_unlock();
+
+	mvwprintw(screen, 1, 0, "%*s", COLS - 1, " ");
+	mvwprintw(screen, 1, 2, "Kernel netfilter flows(%u) for ", flows);
+
+	draw_filter_status("Kernel netfilter flows", flows, skip_lines);
+}
+
+static void draw_proc_entry(struct proc_entry *p)
+{
+	struct ui_table *tbl = &procs_tbl;;
+	char tmp[128];
+
+	ui_table_row_add(tbl);
+
+	/* Application */
+	ui_table_row_col_set(tbl, TBL_PROC_NAME, p->name);
+
+	/* PID */
+	slprintf(tmp, sizeof(tmp), "%.d", p->pid);
+	ui_table_row_col_set(tbl, TBL_PROC_PID, tmp);
+
+	/* Flows */
+	slprintf(tmp, sizeof(tmp), "%.d", p->flows_count);
+	ui_table_row_col_set(tbl, TBL_PROC_FLOWS, tmp);
+
+	/* Bytes Src */
+	bandw2str(p->bytes_src, tmp, sizeof(tmp) - 1);
+	ui_table_row_col_set(tbl, TBL_PROC_BYTES_SRC, tmp);
+
+	/* Rate Src */
+	rate2str(p->rate_bytes_src, tmp, sizeof(tmp) - 1);
+	ui_table_row_col_set(tbl, TBL_PROC_RATE_SRC, tmp);
+
+	/* Bytes Dest */
+	bandw2str(p->bytes_dst, tmp, sizeof(tmp) - 1);
+	ui_table_row_col_set(tbl, TBL_PROC_BYTES_DST, tmp);
+
+	/* Rate Dest */
+	rate2str(p->rate_bytes_dst, tmp, sizeof(tmp) - 1);
+	ui_table_row_col_set(tbl, TBL_PROC_RATE_DST, tmp);
+
+	ui_table_row_show(tbl);
+}
+
+static void draw_procs(WINDOW *screen, struct flow_list *fl,
+		       int skip_lines)
+{
+	struct proc_entry *proc, *tmp;
+	unsigned int line = 4;
+	int skip = skip_lines;
+	int procs = 0;
+
+	rcu_read_lock();
+
+	ui_table_clear(&procs_tbl);
+	ui_table_header_print(&procs_tbl);
+
+	cds_list_for_each_entry_safe(proc, tmp, &proc_list.head, entry) {
+		if (line + 1 >= rows)
+			continue;
+		if (--skip >= 0)
+			continue;
+
+		draw_proc_entry(proc);
+		procs++;
+		line++;
+	}
+
+	rcu_read_unlock();
+
+	draw_filter_status("Processes", procs, skip_lines);
 }
 
 static void draw_help(void)
@@ -1188,26 +1284,27 @@ static void draw_help(void)
 	mvaddnstr(row + 2, col + 2, "Navigation", -1);
 	attroff(A_BOLD | A_UNDERLINE);
 
-	mvaddnstr(row + 4, col + 3, "Up, u, k      Move up", -1);
-	mvaddnstr(row + 5, col + 3, "Down, d, j    Move down", -1);
-	mvaddnstr(row + 6, col + 3, "Left,l        Scroll left", -1);
-	mvaddnstr(row + 7, col + 3, "Right,h       Scroll right", -1);
-	mvaddnstr(row + 8, col + 3, "?             Toggle help window", -1);
-	mvaddnstr(row + 9, col + 3, "q, Ctrl+C     Quit", -1);
+	mvaddnstr(row + 4, col + 3, "TAB           Go to next tab panel", -1);
+	mvaddnstr(row + 5, col + 3, "Up, u, k      Move up", -1);
+	mvaddnstr(row + 6, col + 3, "Down, d, j    Move down", -1);
+	mvaddnstr(row + 7, col + 3, "Left,l        Scroll left", -1);
+	mvaddnstr(row + 8, col + 3, "Right,h       Scroll right", -1);
+	mvaddnstr(row + 9, col + 3, "?             Toggle help window", -1);
+	mvaddnstr(row + 10, col + 3, "q, Ctrl+C     Quit", -1);
 
 	attron(A_BOLD | A_UNDERLINE);
-	mvaddnstr(row + 11, col + 2, "Display Settings", -1);
+	mvaddnstr(row + 12, col + 2, "Display Settings", -1);
 	attroff(A_BOLD | A_UNDERLINE);
 
-	mvaddnstr(row + 13, col + 3, "b     Toggle rate units (bits/bytes)", -1);
-	mvaddnstr(row + 14, col + 3, "a     Toggle display of active flows (rate > 0) only", -1);
-	mvaddnstr(row + 15, col + 3, "s     Toggle show source peer info", -1);
+	mvaddnstr(row + 14, col + 3, "b     Toggle rate units (bits/bytes)", -1);
+	mvaddnstr(row + 15, col + 3, "a     Toggle display of active flows (rate > 0) only", -1);
+	mvaddnstr(row + 16, col + 3, "s     Toggle show source peer info", -1);
 
-	mvaddnstr(row + 17, col + 3, "T     Toggle display TCP flows", -1);
-	mvaddnstr(row + 18, col + 3, "U     Toggle display UDP flows", -1);
-	mvaddnstr(row + 19, col + 3, "D     Toggle display DCCP flows", -1);
-	mvaddnstr(row + 20, col + 3, "I     Toggle display ICMP flows", -1);
-	mvaddnstr(row + 21, col + 3, "S     Toggle display SCTP flows", -1);
+	mvaddnstr(row + 18, col + 3, "T     Toggle display TCP flows", -1);
+	mvaddnstr(row + 19, col + 3, "U     Toggle display UDP flows", -1);
+	mvaddnstr(row + 20, col + 3, "D     Toggle display DCCP flows", -1);
+	mvaddnstr(row + 21, col + 3, "I     Toggle display ICMP flows", -1);
+	mvaddnstr(row + 22, col + 3, "S     Toggle display SCTP flows", -1);
 }
 
 static void draw_header(WINDOW *screen)
@@ -1287,17 +1384,59 @@ static void flows_table_init(struct ui_table *tbl)
 	ui_table_header_color_set(&flows_tbl, COLOR(BLACK, GREEN));
 }
 
+static void procs_table_init(struct ui_table *tbl)
+{
+	ui_table_init(tbl);
+
+	ui_table_pos_set(tbl, 3, 0);
+	ui_table_height_set(tbl, LINES - 3);
+
+	ui_table_col_add(tbl, TBL_PROC_NAME, "NAME", 13);
+	ui_table_col_add(tbl, TBL_PROC_PID, "PID", 7);
+	ui_table_col_add(tbl, TBL_PROC_FLOWS, "FLOWS", 7);
+	ui_table_col_add(tbl, TBL_PROC_BYTES_SRC, "BYTES_SRC", 10);
+	ui_table_col_add(tbl, TBL_PROC_BYTES_DST, "BYTES_DST", 10);
+	ui_table_col_add(tbl, TBL_PROC_RATE_SRC, "RATE_SRC", 14);
+	ui_table_col_add(tbl, TBL_PROC_RATE_DST, "RATE_DST", 14);
+
+	ui_table_col_align_set(tbl, TBL_PROC_BYTES_SRC, UI_ALIGN_RIGHT);
+	ui_table_col_align_set(tbl, TBL_PROC_RATE_SRC, UI_ALIGN_RIGHT);
+	ui_table_col_align_set(tbl, TBL_PROC_BYTES_DST, UI_ALIGN_RIGHT);
+	ui_table_col_align_set(tbl, TBL_PROC_RATE_DST, UI_ALIGN_RIGHT);
+
+	ui_table_col_color_set(tbl, TBL_PROC_NAME, COLOR(YELLOW, BLACK));
+	ui_table_col_color_set(tbl, TBL_PROC_PID, A_BOLD);
+	ui_table_col_color_set(tbl, TBL_PROC_FLOWS, COLOR(YELLOW, BLACK));
+	ui_table_col_color_set(tbl, TBL_PROC_BYTES_SRC, COLOR(RED, BLACK));
+	ui_table_col_color_set(tbl, TBL_PROC_RATE_SRC, COLOR(RED, BLACK));
+	ui_table_col_color_set(tbl, TBL_PROC_BYTES_DST, COLOR(BLUE, BLACK));
+	ui_table_col_color_set(tbl, TBL_PROC_RATE_DST, COLOR(BLUE, BLACK));
+
+	ui_table_header_color_set(tbl, COLOR(BLACK, GREEN));
+}
+
+static void tab_main_on_open(struct ui_tab *tab, enum ui_tab_event_t evt, uint32_t id)
+{
+	if (evt != UI_TAB_EVT_OPEN)
+		return;
+
+	if (id == TAB_FLOWS)
+		draw_flows(screen, &flow_list, skip_lines);
+	else if (id == TAB_PROCS)
+		draw_procs(screen, &flow_list, skip_lines);
+}
+
 static void presenter(void)
 {
 	bool show_help = false;
-	int skip_lines = 0;
-	WINDOW *screen;
+	struct ui_tab *tab_main;
 
 	lookup_init(LT_PORTS_TCP);
 	lookup_init(LT_PORTS_UDP);
 
 	screen = screen_init(false);
 	wclear(screen);
+	halfdelay(1);
 
 	start_color();
 	INIT_COLOR(RED, BLACK);
@@ -1307,6 +1446,14 @@ static void presenter(void)
 	INIT_COLOR(BLACK, GREEN);
 
         flows_table_init(&flows_tbl);
+        procs_table_init(&procs_tbl);
+
+	tab_main = ui_tab_create();
+	ui_tab_event_cb_set(tab_main, tab_main_on_open);
+	ui_tab_pos_set(tab_main, 2, 0);
+	ui_tab_active_color_set(tab_main, COLOR(BLACK, GREEN));
+	ui_tab_entry_add(tab_main, TAB_FLOWS, "Flows");
+	ui_tab_entry_add(tab_main, TAB_PROCS, "Process");
 
 	rcu_register_thread();
 	while (!sigint) {
@@ -1367,6 +1514,9 @@ static void presenter(void)
 			show_option_toggle(ch);
 			do_reload_flows = true;
 			break;
+		case '\t':
+			ui_tab_event_send(tab_main, UI_EVT_SELECT_NEXT);
+			break;
 		default:
 			fflush(stdin);
 			break;
@@ -1377,15 +1527,15 @@ static void presenter(void)
 		if (show_help)
 			draw_help();
 		else
-			draw_flows(screen, &flow_list, skip_lines);
+			ui_tab_show(tab_main);
 
 		draw_footer();
-
-		usleep(80000);
 	}
 	rcu_unregister_thread();
 
 	ui_table_uninit(&flows_tbl);
+	ui_table_uninit(&procs_tbl);
+	ui_tab_destroy(tab_main);
 
 	screen_end();
 	lookup_cleanup(LT_PORTS_UDP);
@@ -1476,38 +1626,58 @@ static int flow_event_cb(enum nf_conntrack_msg_type type,
 	if (sigint)
 		return NFCT_CB_STOP;
 
-	synchronize_rcu();
-	spinlock_lock(&flow_list.lock);
-
 	switch (type) {
 	case NFCT_T_NEW:
-		flow_list_new_entry(&flow_list, ct);
-		break;
+		return flow_list_new_entry(&flow_list, ct);
 	case NFCT_T_UPDATE:
-		flow_list_update_entry(&flow_list, ct);
-		break;
+		return flow_list_update_entry(&flow_list, ct);
 	case NFCT_T_DESTROY:
-		flow_list_destroy_entry(&flow_list, ct);
-		break;
+		return flow_list_del_entry(&flow_list, ct);
 	default:
-		break;
+		return NFCT_CB_CONTINUE;
 	}
+}
 
-	spinlock_unlock(&flow_list.lock);
+static void collector_refresh_procs(void)
+{
+	struct proc_entry *p, *tmp;
 
-	if (sigint)
-		return NFCT_CB_STOP;
+	cds_list_for_each_entry_safe(p, tmp, &proc_list.head, entry) {
+		double sec = (double)time_after_us(&p->last_update) / USEC_PER_SEC;
+		struct flow_entry *n;
 
-	return NFCT_CB_CONTINUE;
+		if (sec < 1)
+			continue;
+
+		bug_on(gettimeofday(&p->last_update, NULL));
+
+		if (!p->flows_count && !proc_exists(p->pid)) {
+			cds_list_del_rcu(&p->entry);
+			call_rcu(&p->rcu, proc_entry_xfree_rcu);
+			continue;
+		}
+
+		p->rate_bytes_src = 0;
+		p->rate_bytes_dst = 0;
+		p->rate_pkts_src = 0;
+		p->rate_pkts_dst = 0;
+
+		cds_list_for_each_entry_rcu(n, &p->flows, proc_head) {
+			p->rate_bytes_src += n->rate_bytes_src;
+			p->rate_bytes_dst += n->rate_bytes_dst;
+			p->rate_pkts_src += n->rate_pkts_src;
+			p->rate_pkts_dst += n->rate_pkts_dst;
+		}
+	}
 }
 
 static void collector_refresh_flows(struct nfct_handle *handle)
 {
 	struct flow_entry *n;
 
-	n = rcu_dereference(flow_list.head);
-	for (; n; n = rcu_dereference(n->next))
+	cds_list_for_each_entry_rcu(n, &flow_list.head, entry) {
 		nfct_query(handle, NFCT_Q_GET, n->ct);
+	}
 }
 
 static void collector_create_filter(struct nfct_handle *nfct)
@@ -1561,9 +1731,6 @@ static int flow_dump_cb(enum nf_conntrack_msg_type type __maybe_unused,
 	if (sigint)
 		return NFCT_CB_STOP;
 
-	synchronize_rcu();
-	spinlock_lock(&flow_list.lock);
-
 	if (!(what & ~(INCLUDE_IPV4 | INCLUDE_IPV6)))
 		goto check_addr;
 
@@ -1616,10 +1783,9 @@ check_addr:
 			goto skip_flow;
 	}
 
-	flow_list_new_entry(&flow_list, ct);
+	return flow_list_new_entry(&flow_list, ct);
 
 skip_flow:
-	spinlock_unlock(&flow_list.lock);
 	return NFCT_CB_CONTINUE;
 }
 
@@ -1651,6 +1817,7 @@ static void *collector(void *null __maybe_unused)
 	struct nfct_handle *ct_event;
 	struct pollfd poll_fd[1];
 
+	proc_list_init(&proc_list);
 	flow_list_init(&flow_list);
 
 	ct_event = nfct_open(CONNTRACK, NF_NETLINK_CONNTRACK_NEW |
@@ -1688,6 +1855,7 @@ static void *collector(void *null __maybe_unused)
 			collector_dump_flows();
 		}
 
+		collector_refresh_procs();
 		collector_refresh_flows(ct_event);
 
 		status = poll(poll_fd, 1, 0);
@@ -1696,18 +1864,16 @@ static void *collector(void *null __maybe_unused)
 				continue;
 
 			panic("Error while polling: %s\n", strerror(errno));
-		} else if (status == 0) {
-			continue;
+		} else if (status != 0) {
+			if (poll_fd[0].revents & POLLIN)
+				nfct_catch(ct_event);
 		}
-
-		if (poll_fd[0].revents & POLLIN)
-			nfct_catch(ct_event);
 	}
 
-	rcu_unregister_thread();
-
 	flow_list_destroy(&flow_list);
-	spinlock_destroy(&flow_list.lock);
+	proc_list_destroy(&proc_list);
+
+	rcu_unregister_thread();
 
 	nfct_close(ct_event);
 
@@ -1717,13 +1883,13 @@ static void *collector(void *null __maybe_unused)
 int main(int argc, char **argv)
 {
 	pthread_t tid;
-	int ret, c, opt_index, what_cmd = 0;
+	int ret, c, what_cmd = 0;
 
 	setfsuid(getuid());
 	setfsgid(getgid());
 
 	while ((c = getopt_long(argc, argv, short_options, long_options,
-	       &opt_index)) != EOF) {
+				NULL)) != EOF) {
 		switch (c) {
 		case '4':
 			what_cmd |= INCLUDE_IPV4;
