@@ -1338,7 +1338,7 @@ static int term_main(const char *ifname, uint64_t ms_interval,
 int main(int argc, char **argv)
 {
 	short ifflags = 0;
-	int c, opt_index, ret, promisc = 0;
+	int c, ret, promisc = 0;
 	unsigned int cpus, top_cpus = 5;
 	uint64_t interval = 1000;
 	char *ifname = NULL;
@@ -1354,7 +1354,7 @@ int main(int argc, char **argv)
 	setfsgid(getgid());
 
 	while ((c = getopt_long(argc, argv, short_options, long_options,
-	       &opt_index)) != EOF) {
+				NULL)) != EOF) {
 		switch (c) {
 		case 'h':
 			help();
